#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""RuSocSci module for utilities, like listing USB to serial devices and connecting to them.

# Copyright (C) 2013 Wilbert van Ham, Radboud University Nijmegen
# Distributed under the terms of the GNU General Public License (GPL) version 3 or newer.

Known issues:
    - All usb2serial devices are detected. The list of joysticks therefore also contains
      buttonboxes.

Typical usage::

    from psychopy import visual
    from rusocsci import joystick
    import time

    win = visual.Window([400,400])

    nJoys = joystick.getNumJoysticks()#to check if we have any
    id=0
    joy = joystick.Joystick(id)#id must be <= nJoys-1

    while True:#while presenting stimuli
        joy.getX()
        time.sleep(1)
"""
import sys, serial, time, os, re, logging, struct
if sys.platform == "win32":
	import _winreg

# our buttonboxand joystick have id 0403:6001 from its UART IC
# make sure you have the pyusb module installed
# for MS Windows you may need this: http://sourceforge.net/apps/trac/libusb-win32/wiki
def serialList():
	"Get a list of USB to serial devices connected."
	if sys.platform == "win32":
		try:
			# find number of FTDI USB to serial devices attached
			reg = _winreg.ConnectRegistry(None, _winreg.HKEY_LOCAL_MACHINE)
			key = _winreg.CreateKeyEx(reg, r"SYSTEM\CurrentControlSet\services\FTDIBUS\Enum", 0, _winreg.KEY_READ)
			count = _winreg.QueryValueEx(key, "Count")[0]
		except Exception as e:
			logging.error("Could not find USB joystick, no FTDI bus, \n{}".format(e))
			return ""
		
		if count == 0:
			logging.error("No USB joystick connected.")
			return []
		if count > 1:
			logging.debug("Multiple USB serial connected, last connected is first in list.")
			
		devices = []
		for i in range(count-1, -1, -1):
			value = _winreg.QueryValueEx(key, str(i))[0]
			devices.append(value)
		logging.debug("USB to serial devices: {}".format(devices))
		
		ports = []
		for device in devices:
			id = device[-8:] #for instance 
			# find port, the following line gives an error in windows without administrative priviliges
			key = _winreg.CreateKey(reg, r"SYSTEM\CurrentControlSet\Enum\FTDIBUS\VID_0403+PID_6001+"+id+r"A\0000\Device Parameters")
			try:
				value = _winreg.QueryValueEx(key, "PortName")[0]
				ports.append(value)
			except Exception as e:
				logging.info("Could not find USB serial device, no value for PortName: {}\n".format(e))
		logging.debug("USB to serial interfaces: {}".format(ports))
		return ports
	elif sys.platform == "darwin":
		s = os.popen('ls -t /dev/tty.usbserial*').read().rstrip()
		ports = s.split()
		if len(ports)==0:
			logging.info("Could not find USB serial device. Install the FTDI VCP driver and plug in the device.")
		if len(ports) > 1:
			logging.debug("Multiple USB serial devices connected, last connected is first in list.")
		logging.debug("USB to serial interfaces: {}".format(ports))
		return ports
	else:
		# http://stackoverflow.com/questions/2530096/how-to-find-all-serial-devices-ttys-ttyusb-on-linux-without-opening-them
		# /sys/bus/usb/devices/
		# /sys/devices/pci0000:00/0000:00:1d.0/usb2/2-1/2-1.6/serial
		#s = os.popen('lsusb -d 0403:6001').read().rstrip()
		s = os.popen('ls -t /dev/serial/by-id/*FTDI*').read()
		ports = s.split()
		if len(ports)==0:
			logging.info("Could not find USB serial device.")
		if len(ports) > 1:
			logging.debug("Multiple USB serial devices connected, last connected is first in list.")
		logging.debug("USB to serial interfaces: {}".format(ports))
		return ports


def open(id=0, port=None):
	""" id is position in list (0 is latest), port is 'COM1', /dev/tty, 0 or something similar
	Note that port can also be a real serial port (non USB).
	returns serial connection and id string. """
	device = None
	if port == None:
		ports = serialList()
		if len(ports)>id:
			port = ports[id]
		else:
			logging.error("id {} invalid, number of USB to serial ports found: {}".format(id, len(ports)))
			return [None, ""]
	if port == None or port == "":
		logging.error("No USB to serial port found not found.")
		return [None, ""]
	else:
		logging.debug("Connecting to USB to serial port: {}".format(port))
		
	try:
		device = serial.Serial(port, baudrate=115200, parity='N', timeout = 0.0)  # open serial port
	except Exception as e:
		logging.error("Could not connect to serial port {}: \n{}".format(port, e))
		return None
		
	# reset connection (needed for Linux if port was opened before)
	device.setDTR(True)
	device.setDTR(False)
	
	#idString = "joystick streaming angle, Ready!" + "0d0a".decode("hex")
	# collect byes up to "!\x0d\x0a" that identify the type of device
	beginTime = time.time()
	bytesRead = ""
	while len(bytesRead) < 3 or bytesRead[-3:] != "!\x0d\x0a":
		if time.time() > beginTime + 3:
			logging.error("USB to serial timeout")
			device = None
			return [None, "USB to serial timeout"]
		bytes = device.read()
		#if len(bytes) > 0:
			#print("bytes: #{}#".format(bytes))
		#if len(bytesRead) > 2:
			#print("bytesRead: #{}#".format(bytesRead[-3:]))
		bytesRead += bytes
		
	return [device, bytesRead[:-2]]





#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""RuSocSci module for the red joystick

Copyright (C) 2013 Wilbert van Ham, Radboud University Nijmegen
Distributed under the terms of the GNU General Public License (GPL) version 3 or newer.

Known issues:
    - All usb2serial devices are detected. The list of joysticks therefore also contains
      buttonboxes.

Typical usage::

    from psychopy import visual
    from rusocsci import joystick
    import time

    win = visual.Window([400,400])

    nJoys = joystick.getNumJoysticks()#to check if we have any
    id=0
    joy = joystick.Joystick(id)#id must be <= nJoys-1

    while True:#while presenting stimuli
        joy.getX()
        time.sleep(1)
"""
import sys, serial, time, os, re, logging, struct
import utils

# our buttonboxand joystick have id 0403:6001 from its UART IC
# make sure you have the pyusb module installed
# for MS Windows you may need this: http://sourceforge.net/apps/trac/libusb-win32/wiki
def getNumJoysticks():
	""""Return a count of the number of joysticks available. 
	This really returns a count of all usb to serial devices"""
	return len(utils.serialList())

class Joystick():
	def __init__(self, id=0, port=None):
		self.x = 126 # angle range is [51..201]
		[self._device, idString] = utils.open(id, port)

		if idString == "joystick streaming angle, Ready!":
			logging.debug("Device is a joystick ({}): {}".format(len(idString), idString))
		else:
			logging.error("Device is NOT a joystick ({}): {}".format(len(idString), idString))

	def __del__(self):
		if self._device:
			self._device.close()

	def getNumHats(self):
		"Get the number of hats on this joystick"
		return 0
	def getX(self):
		"Returns the value on the X axis (equivalent to joystick.getAxis(0))"
		if self._device == None:
			logging.error("Joystick not initialized")
			return -1
		self._device.timeout = 0
		c = self._device.read()
		nBytes = 0
		while len(c) != 0:
			self.x = struct.unpack('B', c)[0]
			nBytes+=1
			c = self._device.read()
		logging.debug("read {} bytes, ending in {}".format(nBytes, self.x))
		return self.x

	def getAllAxes(self):
		"Get a list of all current axis values"
		return [self.getX()]
	def getNumAxes(self):
		"Returns the number of joystick axes found"
		return 1
	def getAxis(self, axisId):
		"Get the value of an axis by an integer id (from 0 to number of axes-1)"
		return self.getX()
	
	def clearEvents():
		if _device == None:
			event.flushInput(keyboard)
		else:
			self._device.flushInput()
		



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""RuSocSci module for the BITSI buttonbox

Copyright (C) 2013 Wilbert van Ham, Radboud University Nijmegen
Distributed under the terms of the GNU General Public License (GPL) version 3 or newer.
"""
from psychopy import core, visual, monitors, event
import sys, serial, time, os, re, logging, glob
import utils

# our buttonbox has id 0403:6001 fro  its UART IC
# make sure you have the pyusb module installed
# for window you may need this: http://sourceforge.net/apps/trac/libusb-win32/wiki

class Buttonbox():
	def __init__(self, id=0, port=None):
		[self._device, idString] = utils.open(id, port)
		if idString == "BITSI mode, Ready!" or idString == "BITSI event mode, Ready!":
			logging.debug("Device is a BITSI buttonbox ({}): {}".format(len(idString), idString))
		else:
			logging.error("Device is NOT a BITSI buttonbox ({}): {}".format(len(idString), idString))

	def __del__(self):
		if self._device:
			self._device.close()
				
	def clearEvents(self):
		if self._device == None:
			event.flushInput(keyboard)
		else:
			self._device.flushInput()
		
	def getButtons(self, buttonList=None):
		"""Returns a list of buttons that were pressed on the buttonbox. Falls back to 
		keyboard if there is no buttonbox.

		:Parameters:
		keyList : **None** or []
		Allows the user to specify a set of buttons to check for.
		Only keypresses from this set of keys will be removed from the keyboard buffer.
		If the keyList is None all keys will be checked and the key buffer will be cleared
		completely. NB, pygame doesn't return timestamps (they are always 0)
		here is no timestamp in our buttonbox. Use buttonbox.waitkeys if you want timestamps.

		:Author:
			- 2013 written by Wilbert van Ham
		"""
		if self._device == None:
			try:
				return event.getKeys(keyList=[b.lower() for b in buttonList])
			except:
				logging.error("No buttonbox connected and no psychopy.event imported for fallback")
				return []
		self._device.timeout = 0
		cList = self._device.read(1024)
		#if len(cList)>0:
			#logging.debug("read {} bytes: {}".format(len(cList), cList))
		cListSelected = []
		for c in cList:
			if buttonList==None or c in buttonList:
				cListSelected.append(c)
		return cListSelected
				
	def waitButtons(self, maxWait=float("inf"), buttonList=None, timeStamped=False):
		"""
		Same as `~socsci.buttonbox.getButtons`, but halts everything (including drawing) while awaiting
		input from buttonbox. Implicitly clears buttonbox, so any preceding buttonpresses will be lost.

		:Parameters:
		maxWait : any numeric value.
		Maximum number of seconds period and which buttons to wait for. 
		Default is float('inf') which simply waits forever.
		buttonList:
		List of one character strings containing the buttons to react to.
		All other button presses will be ignored. Notethat for BITSI 
		buttonboxes the buttons are identified by capital letters upon press
		and by lower case letters upon release.

		Returns None if times out.
		"""
		if self._device == None:
			try:
				return event.waitKeys(maxWait=maxWait, keyList=[b.lower() for b in buttonList], timeStamped=timeStamped)
			except:
				logging.error("No buttonbox connected and no psychopy.event imported for fallback")
				return []
				
		self._device.flushInput()
		t = time.time()
		while maxWait - (time.time() - t) > 0:
			if maxWait == float("inf"):
				self._device.timeout = None
			else:
				self._device.timeout = maxWait - (time.time() - t)
			c = self._device.read(1)
			if buttonList==None or c in buttonList:
				# return
				break
			else:
				# discard
				c = None
		if hasattr(timeStamped, 'timeAtLastReset'):
			return [(c, time.time() - timeStamped.timeAtLastReset)]
		elif timeStamped:
			# return as a one item list to mimic getButtons behaviour
			return [(c, time.time() - t)]
		else:
			return c
			
	def sendMarker(self, leds=[False,False,False,False,False,False,False,False], val=None):
		setLeds(self, leds, val)
	def setLeds(self, leds=[False,False,False,False,False,False,False,False], val=None):
		"""Set buttonbox LEDs to a certain pattern """
		if self._device == None:
			logging.error("No buttonbox connected")
			return 
		if val == None:
			val = 0
			for i in range(8):
				if len(leds)>i:
					if leds[i]:
						val += 1<<i
				else:
					break
		self._device.write(chr(val))
		
	def waitLeds(self, leds=[False,False,False,False,False,False,False,False], wait=1.0, val=None):
		"""Set buttonbox LEDs to a certain pattern and wait a while"""	
		if self._device == None:
			logging.error("No buttonbox connected")
			return 
		setLeds(leds, val)
		time.sleep(wait)
		setLeds()




#!/usr/bin/python
import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
	return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
	name = "RuSocSci",
	version = "0.6.4",
	install_requires = "pyserial",
	author = "Wilbert van Ham",
	author_email = "W.vanHam@socsci.ru.nl",
	description = "Support package for Radboud University Nijmegen"\
			"Faculty of Social Sciences hardware with PsychoPy-like API.",
	license = "GPLv3+",
	keywords = "hardware",
	url = "http://www.socsci.ru.nl/~wilberth/python/hardware.html",
	packages=['rusocsci'],
	long_description=read('README'),
	classifiers=[
		"Development Status :: 4 - Beta",
		"License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
	],
)
